package linearalgebra;

import linearalgebra.*;
import linearalgebra.exception.UnsupportedOperationError;

public class IdentityMatrix implements linearalgebra.MatrixInterface {	
	private int size;
	public IdentityMatrix( int size ) {
		this.size = size;
	}
	
    public boolean equals(Object o) {
    	boolean rc = false;
    	if( o instanceof MatrixInterface ) {
    		MatrixInterface m = (MatrixInterface)o;
    		rc = m.isIdentity() && m.getRowCount() == size;
    	}
        return rc;
    }

    public MatrixInterface factorL(PermutationMatrix p) {
        return factorL(p, true);
    }

	public MatrixInterface factorL(PermutationMatrix p, boolean partialPivot) {
        return new IdentityMatrix(size);
    }

    public MatrixInterface factorU(PermutationMatrix p) {
        return new IdentityMatrix(size);
    }

    public int getColumnCount() {
        return size;
    }

    public MatrixInterface factorInverse() {
        return new IdentityMatrix(size);
    }
    
    public MatrixInterface getTranspose() {
    	return new IdentityMatrix(size);
    }

    public int getRowCount() {
        return size;
    }

    public boolean isIdentity() {
        return true;
    }

    public boolean isLowerLeftTriangular() {
        return true;
    }

    public boolean isUpperRightTriangular() {
        return true;
    }
    
	public boolean isSymmetric() {
		return true;
	}

    public boolean isPositiveDefinite() {
    	return true;
    }

    public void swapRows(int row1, int row2) {
        throw new linearalgebra.exception.UnsupportedOperationError();
    }

    public void swapRows(PermutationMatrix p) {
        throw new linearalgebra.exception.UnsupportedOperationError();
    }
	
	public boolean hasRowPivot( int row ) {
		return true;
	}
	
	public boolean hasColumnPivot( int col ) {
		return true;
	}
	
	public int getRowLeftPivotIndex( int row ) {
		return row;
	}
	
	public int getRowRightPivotIndex( int row ) {
		return row;
	}
	
	public int getColumnTopPivotIndex( int col ) {
		return col;
	}
	
	public int getColumnBottomPivotIndex( int col ) {
		return col;
	}
    
    public MatrixInterface getProjectionMatrix() {
    	throw new UnsupportedOperationError();
    }
    
    public MatrixInterface getOrthogonalProjectionMatrix() {
    	throw new UnsupportedOperationError();
    }
    
    public MatrixInterface getReflectionMatrix() {
    	throw new UnsupportedOperationError();
    }
    
    public boolean isIdempotent() {
    	throw new UnsupportedOperationError();
    }

	public boolean isOrthogonal() {
		return true;
	}
}