package linearalgebra;

import linearalgebra.exception.UnsupportedOperationError;

public class PermutationMatrix implements linearalgebra.MatrixInterface {
	private final int[] rows;
	
	public PermutationMatrix( int size ) {
		rows = new int[ size ];
		setToIdentity();
	}
	
	private PermutationMatrix( int[] rows ) {
		this.rows = rows;
	}
	
	private void setToIdentity() {
		for( int i = 0; i < rows.length; i++ ) {
			rows[i] = i;
		}
	}
	
    public boolean equals(Object o) {
    	boolean rc = false;
    	if( o instanceof PermutationMatrix ) {
    		PermutationMatrix p =(PermutationMatrix)o;
    		if( rows.equals(p.rows) ) {
    			rc = true;
    		}
    	}
        return rc;
    }

    public MatrixInterface factorL(PermutationMatrix p) {
    	return factorL( p, true );
    }

    public MatrixInterface factorL(PermutationMatrix p, boolean partialPivot ) {
    	System.arraycopy(rows, 0, p.rows, 0, rows.length);
    	setToIdentity();
        return new PermutationMatrix(rows.length);
    }

    public MatrixInterface factorU(PermutationMatrix p) {
    	throw new UnsupportedOperationError();
    }

    public int getColumnCount() {
        return rows.length;
    }

    public MatrixInterface factorInverse() {
        MatrixInterface rc = getTranspose();
        setToIdentity();
        return rc;
    }
    
    public MatrixInterface getTranspose() {
    	int[] t = new int[rows.length];
    	for( int i = 0; i < rows.length; i++ ) {
    		t[ rows[i] ] = i;
    	}
    	return new PermutationMatrix(t);
    }

    public int getRowCount() {
        return rows.length;
    }

    public boolean isIdentity() {
    	for( int i = 0; i < rows.length; i++ ) {
    		if( rows[i] != i ) return false;
        }    	
        return true;
    }

    public boolean isLowerLeftTriangular() {
        return isIdentity();
    }

    public boolean isUpperRightTriangular() {
        return isIdentity();
    }
    
    public boolean isSymmetric() {
    	for( int i = 0; i < rows.length; i++ ) {
    		if( i != rows[i] ) {
    			return false;
    		}
    	}
    	return true;
    }

    public boolean isPositiveDefinite() {
    	return isSymmetric();
    }

    public void swapRows(int row1, int row2) {
    	int temp = rows[row1];
    	rows[row1] = rows[row2];
    	rows[row2] = temp;
    }

    public void swapRows(PermutationMatrix p) {
    	if( p.rows.length != rows.length ) {
    		throw new linearalgebra.exception.SizeMismatchError("Permuation size does not match matrix size.");
    	}
    	int[] newrows = new int[ rows.length ];
    	for( int i = 0; i < rows.length; i++ ) {
    		newrows[i] = p.rows[ rows[i] ];
        }   	
    }
	
	public boolean hasRowPivot( int row ) {
		throw new linearalgebra.exception.NoPivotError();
	}

	public boolean hasColumnPivot( int col ) {
		throw new linearalgebra.exception.NoPivotError();
	}
	
	public int getRowLeftPivotIndex( int row ) {
		throw new linearalgebra.exception.NoPivotError();
	}
	
	public int getRowRightPivotIndex( int row ) {
		throw new linearalgebra.exception.NoPivotError();
	}
	
	public int getColumnTopPivotIndex( int col ) {
		throw new linearalgebra.exception.NoPivotError();
	}
	
	public int getColumnBottomPivotIndex( int col ) {
		throw new linearalgebra.exception.NoPivotError();
	}
    
    public int getRowTranspose( int row ) {
    	return rows[row];
    }
    
    public String toString() {
		StringBuffer rc = new StringBuffer();
		for( int i = 0; i < getRowCount(); i++ ) {
			for( int j = 0; j < getColumnCount(); j++ ) {
				if( j == rows[i] ) {
					rc.append("1\t");
				} else {
					rc.append("0\t");
				}
			}
			rc.append("\n");
		}
		return rc.toString();
    }
    
    public MatrixInterface getProjectionMatrix() {
    	throw new UnsupportedOperationError();
    }
    
    public MatrixInterface getOrthogonalProjectionMatrix() {
    	throw new UnsupportedOperationError();
    }
    
    public MatrixInterface getReflectionMatrix() {
    	throw new UnsupportedOperationError();
    }
    
    public boolean isIdempotent() {
    	throw new UnsupportedOperationError();
    }

	public boolean isOrthogonal() {
		return true;
	}
}