package linearalgebra.numeric;

import linearalgebra.MatrixInterface;

public class Matrix extends AbstractMatrix {
	private double[][] m_matrix = {};
	
	public Matrix( int rows, int cols ) {
		super( rows, cols );
		m_matrix = new double[rows][cols];
	}
	
	public Matrix( int identity ) {
		this( identity, identity );
		for( int i = 0; i < identity; i++ ) {
			setElement(i, i, 1.0);
		}
	}
	
	public Matrix( Matrix m ) {
		this( m.m_matrix );
	}
	
	public Matrix( double[][] m ) {
		super( m.length, (m.length > 0) ? m[0].length : 0 );
		m_matrix = new double[getRowCount()][];
		for( int i = 0; i < getRowCount(); i++ ) {
			m_matrix[i] = new double[ getColumnCount() ];
			for( int j = 0; j < getColumnCount(); j++ ) {
				m_matrix[i][j] = m[i][j];
			}
		}
	}
	
	public Object clone() {
		Matrix rc = (Matrix)super.clone();
		rc.m_matrix = new double[m_matrix.length][];
		for( int i = 0; i < m_matrix.length; i++ ) {
			rc.m_matrix[i] = m_matrix[i].clone();
		}
		return rc;
	}

	public final void swapRows( int row1, int row2 ) {
		double[] temp = m_matrix[row1];
		m_matrix[row1] = m_matrix[row2];
		m_matrix[row2] = temp;
	}
	
	public void swapRows( linearalgebra.PermutationMatrix p ) {
		double[][] temp = new double[getRowCount()][];
		for( int i = 0; i < getRowCount(); i++ ) {
			temp[p.getRowTranspose(i)] = m_matrix[ i ];
		}
		System.arraycopy( temp, 0, m_matrix, 0, temp.length );
	}
		
	public MatrixInterface getTranspose() {
		Matrix rc = new Matrix( getColumnCount(), getRowCount() );
		for( int i = 0; i < getRowCount(); i++ ) {
			NumericVectorInterface v = getNumericRowView(i).getNumericVectorView();
			rc.getNumericColumnView(i).getNumericVectorView().setValue(v);
		}
		return rc;
	}

	public double getElement( int row, int col ) {
		return m_matrix[row][col];
	}
	
	public void setElement( int row, int col, double element ) {
		m_matrix[row][col] = element;
	}
	
	public void scaleElement( int row, int col, double scalar ) {
		m_matrix[row][col] *= scalar;
	}
	
	public void incrementElement( int row, int col, double value ) {
		m_matrix[row][col] += value;
	}
	
	public boolean isIdempotent() {
		NumericMatrixInterface a2 = product(this);
		return equals(a2);
	}
}