package linearalgebra.symbolic;

import linearalgebra.symbolic.*;
import linearalgebra.exception.UnboundValueError;

public abstract class PairOperator extends linearalgebra.symbolic.AbstractUnknown {
	private SymbolicValue m_left;
	private SymbolicValue m_right;
	
	public PairOperator( SymbolicValue left, SymbolicValue right ) {
		m_left = left;
		m_right = right;
		m_left.addParent(this);
		m_right.addParent(this);
	}
	
	public final void setChild( int index, SymbolicValue v ) {
    	switch( index ) {
    		case 0: {
    			if( m_left != null ) {
    				m_left.removeParent(this);
    			}
    			m_left = v;
    			if( m_left != null ) {
    				m_left.addParent(this);
    			}
    		} break;
    		case 1: {
    			if( m_right != null ) {
    				m_right.removeParent(this);
    			}
    			m_right = v;
    			if( m_right != null ) {
    				m_right.addParent(this);
    			}
    		} break;
    		default: throw new ArrayIndexOutOfBoundsException();
    	}
	}
	
    public final SymbolicValue getChild(int index) {
    	switch( index ) {
    		case 0: return m_left; 
    		case 1: return m_right;
    		default: throw new ArrayIndexOutOfBoundsException();
    	}
    }

    public final int getChildCount() {
        return 2;
    }
    
    public double getNumericValue() throws UnboundValueError {
    	return doOperation(
    		m_left.getNumericValue(),
    		m_right.getNumericValue()
    	);
    }
    
    public String toString() {
    	StringBuffer rc = new StringBuffer();
    	appendParenthesisString(m_left, rc);
    	rc.append(" ");
    	rc.append(getOperator());
    	rc.append(" ");
    	appendParenthesisString(m_right, rc);
    	return rc.toString();
    }
    
	protected void reorderChildren(int[] order) {
		SymbolicValue[] children = new SymbolicValue[order.length];
		if( order.length == 2 ) {
			children[order[0]] = m_left;
			children[order[1]] = m_right;
			m_left = children[0];
			m_right = children[1];
			subTreeChanged();
		} else if( order.length == 3 ) {
			int index = 0;
			if( getChild(0).getClass().equals(getClass()) ) {	// make sure this is the same operator
				children[order[index++]] = getChild(0).getChild(0);
				children[order[index++]] = getChild(0).getChild(1);
				children[order[index++]] = getChild(1);
			} else if( getChild(1).getClass().equals(getClass()) ) {
				children[order[index++]] = getChild(0);
				children[order[index++]] = getChild(1).getChild(0);
				children[order[index++]] = getChild(1).getChild(1);
			} else {
				throw new Error();
			}
			setChild( 0, children[0] );
			setChild( 1, createNewInstance( children[1], children[2] ) );
			subTreeChanged();
		} else if( order.length == 4 ) {
			int index = 0;
			if( getChild(0).getClass().equals(getClass()) && getChild(1).getClass().equals(getClass()) ) {
				children[order[index++]] = getChild(0).getChild(0);
				children[order[index++]] = getChild(0).getChild(1);
				children[order[index++]] = getChild(1).getChild(0);
				children[order[index++]] = getChild(1).getChild(1);
			} else {
				throw new Error();
			}
			setChild( 0, createNewInstance( children[0], children[1] ) );
			setChild( 1, createNewInstance( children[2], children[3] ) );
		}
	}
	
    protected abstract double doOperation( double left, double right );
    protected abstract String getOperator();
    protected abstract PairOperator createNewInstance(SymbolicValue left, SymbolicValue right);
}